-------------------------------------------------
-- Eternity alpha 0.5
-- Licence: GPL
-- Copyright (c) 2011 Caldas Lopes aka Linux-man
-------------------------------------------------

ray = {}
ray.__index = ray

function ray.create(x, y)
	local temp = {}
	setmetatable(temp, ray)
	temp.pos = vector(rnd:random(x-w, x+w),
		rnd:random(y-h, y+h))
	temp.v = vector(0, 0)
	return temp
end

function ray:update(dt)
	self.pos = self.pos - player.v * u.scale * dt
	self.angle = player.angle
	if self.pos.x <  - w then
		self.pos.x = self.pos.x + w * 2
		self.pos.y = self.pos.y + rnd:random(-10, 10)
	elseif self.pos.x >  w then
		self.pos.x = self.pos.x - w * 2
		self.pos.y = self.pos.y + rnd:random(-10, 10)
	end
	if self.pos.y <  - h then
		self.pos.y = self.pos.y + h * 2
		self.pos.x = self.pos.x + rnd:random(-10, 10)
	elseif self.pos.y >  h then
		self.pos.y = self.pos.y - h * 2
		self.pos.x = self.pos.x + rnd:random(-10, 10)
	end
end

function ray:draw()
	local p2 = self.pos + player.v / player.v_max * u.scale * 256
	love.graphics.line(self.pos.x, self.pos.y, p2.x, p2.y)
end

star = {}
star.__index = star

function star:init()
	self.color = {rnd:random(128, 255), rnd:random(128, 255), rnd:random(128, 255), 255}
	self.scale = rnd:random() * 6
	self.delta = 0.01 + rnd:random() * 0.09 --0.9 + rnd:random() * 0.09
end

function star.create(x, y)
	local temp = {}
	setmetatable(temp, star)
	temp.pos = vector(rnd:random(x-w / 2, x+w / 2),
		rnd:random(y-h / 2, y+h / 2))
	temp.v = vector(0, 0)
	temp:init()
	return temp
end

function star:update(dt)
	self.v = -player.v * self.delta
	self.pos = self.pos + self.v * dt
	if self.pos.x <  - w / 2 then
		self.pos.x = self.pos.x + w
		self.pos.y = self.pos.y + rnd:random(-10, 10)
		self:init()
	elseif self.pos.x >  w / 2 then
		self.pos.x = self.pos.x - w
		self.pos.y = self.pos.y + rnd:random(-10, 10)
		self:init()
	end
	if self.pos.y <  - h / 2 then
		self.pos.y = self.pos.y + h
		self.pos.x = self.pos.x + rnd:random(-10, 10)
		self:init()
	elseif self.pos.y >  h / 2 then
		self.pos.y = self.pos.y - h
		self.pos.x = self.pos.x + rnd:random(-10, 10)
		self:init()
	end
end

function star:draw()
	love.graphics.setColor(self.color)
	love.graphics.setPoint(self.scale * u.scale , 'smooth')
	love.graphics.point(self.pos.x, self.pos.y)
end

nebula = {}
nebula.__index = nebula

function nebula:init()
	self.color = {rnd:random(64, 192), rnd:random(64, 192), rnd:random(64, 192), rnd:random(64, 128)}
	self.r = rnd:random() * math.pi*2
	self.scale = vector(rnd:random() * 12 - 6, rnd:random() * 12 - 6)
	self.delta = 0.5 + rnd:random() * 0.3
end

function nebula.create(x, y)
	local temp = {}
	setmetatable(temp, nebula)
	temp.pos = vector(rnd:random(x-w * 8, x+w * 8), --*2
		rnd:random(y-h * 8, y+h * 8))
	temp.v = vector(0, 0)
	temp:init()
	return temp
end

function nebula:update(dt)
	self.v = player.v * self.delta
	self.pos = self.pos + self.v * dt
	if self.pos.x < player.pos.x - w * 10 then --*2
		self.pos.x = self.pos.x + w * 20 --*4
		self.pos.y = self.pos.y + rnd:random(-100, 100)
		self:init()
	elseif self.pos.x > player.pos.x + w * 10 then
		self.pos.x = self.pos.x - w * 20
		self.pos.y = self.pos.y + rnd:random(-100, 100)
		self:init()
	end
	if self.pos.y < player.pos.y - h * 10 then
		self.pos.y = self.pos.y + h * 20
		self.pos.x = self.pos.x + rnd:random(-100, 100)
		self:init()
	elseif self.pos.y > player.pos.y + h * 10 then
		self.pos.y = self.pos.y - h * 20
		self.pos.x = self.pos.x + rnd:random(-100, 100)
		self:init()
	end
end

function nebula:draw()
	love.graphics.setColor(self.color)
	love.graphics.draw(neb, self.pos.x, self.pos.y, self.r, self.scale.x, self.scale.y)
	love.graphics.setColor(255, 255, 255, 255)
end
