local log256_ = math.log( 256 )
local function log256( n )
    return math.log( n ) / log256_
end
local function numToBytes( n )
    local len = math.ceil( log256( n + 1 ) )
    local b = { string.char( len ) }
    for i = 2, len + 1 do
        b[ i ] = string.char( n % 256 )
        n = math.floor( n / 256 )
    end
    return table.concat( b )
end
function compress( data )
    local dictionarySize = 256
    local dictionary = {}
    for i = 0, 255 do
        dictionary[ string.char( i ) ] = i
    end
    local out = {}
    local outSize = 1

    local a = ""
    for i = 1, #data do
        local b = data:sub( i, i )
        local ab = a .. b
        if dictionary[ ab ] then
            a = ab
        else
            dictionary[ ab ] = dictionarySize
            dictionarySize = dictionarySize + 1
            out[ outSize ] = numToBytes( dictionary[ a ] )
            outSize = outSize + 1
            a = b
        end
    end
    out[ outSize ] = numToBytes( dictionary[ a ] )

    return table.concat( out )
end

local function bytesToNum( b )
    local n = 0
    local pow = 1
    for i = 1, #b do
        n = n + string.byte( b, i, i ) * pow
        pow = pow * 256
    end
    return n
end
function decompress( data )
    local dictionarySize = 256
    local dictionary = {}
    for i = 0, 255 do
        dictionary[ i ] = string.char( i )
    end
    local a = dictionary[ bytesToNum( data:sub( 2, 2 ) ) ]
    local out = { a }
    local outSize = 2

    local i = 3
    while i < #data do
        local n = string.byte( data:sub( i, i ) )
        local v = dictionary[ bytesToNum( data:sub( i + 1, i + n ) ) ] or ( a .. a:sub( 1, 1 ) )
        out[ outSize ] = v
        outSize = outSize + 1
        dictionary[ dictionarySize ] = a .. v:sub( 1, 1 )
        dictionarySize = dictionarySize + 1
        a = v
        i = i + n + 1
    end

    return table.concat( out )
end