-------------------------------------------------
-- Eternity alpha 0.5
-- Licence: GPL
-- Copyright (c) 2011 Caldas Lopes aka Linux-man
-------------------------------------------------
--function object:initialize(parent, x, y, vx, vy, hull, angle, scale, r, g, b, img)
--function asteroid:initialize(parent, x, y)
--function planet:initialize(parent, d, angle, scale, r, g, b, img)
--function sun:initialize(d, angle, scale, r, g, b)


object = class('object')

function object:initialize(name, pos, v, hull, angle, scale, r, g, b, img)
	self.name = name
	self.status = ''
	self.pos = pos
	self.v = v
	self.hull = hull
	self.angle = angle
	self.img = img
	if self.img == nil then self.off = 0
	else self.off = self.img:getHeight() / 2 end
	self.scale = scale
	self.color = {r, g, b, 255}
	self.destroyed = false
	self.draw_dist = 10000^2
	self.update_dist = 1200000^2
	self.destroy_dist = 1100000^2
	self.engine = love.graphics.newParticleSystem(eng_img, 500)
	self.engine:setLifetime(0.5)
	self.engine:setEmissionRate(1000)
	self.engine:setSpeed(300 * self.scale, 400 * self.scale)
	self.engine:setSize(1, 5)
	self.engine:setColor(255, 255, 255, 255, self.color[1], self.color[2], self.color[3], 0)
	self.engine:setParticleLife(0.5)
	self.engine:setSpread(math.pi * 2)
	self.engine:stop()
end

function object:update(dt)
	if (self.pos - player.pos):len2() > self.destroy_dist then
		self:destroy()
		self.engine:stop()
	end
	if (self.pos - player.pos):len2() > self.update_dist then return end
	self.engine:update(dt)
	if self.destroyed == false then
		self.pos = self.pos + self.v * dt
	end
 end

function object:draw()
	if (self.pos - player.pos):len2() > self.draw_dist then return end
--[[
	love.graphics.push()
	love.graphics.translate((self.pos):unpack())
	if self.destroyed == false and player.scan_mode > 3 then --hull bar
		love.graphics.rectangle('fill', 0, 0 - 2 * self.off, 20, 6)
		love.graphics.setColor(255 ,0, 0, 255)
		love.graphics.rectangle('fill', 0, 0 - 2 * self.off, self.hull / (self.scale * 10) * 20, 6)
		love.graphics.setColor(255 ,255, 255, 255)
	end
	love.graphics.pop()
--]]
	love.graphics.setColor(self.color)
	if self.destroyed == false then love.graphics.draw(self.img, self.pos.x, self.pos.y, self.angle, self.scale, self.scale, self.off, self.off) end
	--love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(self.engine, self.pos.x, self.pos.y)
end

function object:hit(hit)
	self.hull = math.max(0, self.hull - hit)
	if self.hull == 0 then self:destroy() end
end

function object:destroy()
	if self.destroyed == false then
		self.engine:start()
		self.destroyed = true
	end
end

asteroid = class('asteroid', object)

function asteroid:initialize(pos)
	local angle =  rnd:random () * 2 * math.pi - math.pi
	local scale = 0.5 + rnd:random () * 1.5
	local hull = 40 * scale
	local r, g, b = rnd:random(32, 128), rnd:random(32, 64), rnd:random(32, 128)
	local v = vector(rnd:random ()*400 - 200, rnd:random ()*400 - 200)
	local img = asteroid_img[rnd:random(#asteroid_img)]
	object.initialize(self, '', pos, v, hull, angle, scale, r, g, b, img)
	self.type = 'asteroid'
end

function asteroid:update(dt)
	object.update(self, dt)
	self.angle = self.angle + math.abs(self.angle) / self.angle * rnd:random() / 2 * dt
end

field = class('field', asteroid)

function field:initialize(pos)
	asteroid.initialize(self, pos)
	self.type = 'field'
	self.destroy_dist = 1400000^2
end

planet = class('planet', object)

function planet:initialize(ss, planets_n, planet_n)
	rnd:randomseed(ss * 10 + planet_n)
	local name = create_name()
	local d, scale = (planet_n + rnd:random()) * (1000000 / planets_n), 1 + rnd:random () * 3
	local r, g, b, img = rnd:random(32, 128), rnd:random(32, 128), rnd:random(32, 128), planet_img[rnd:random(#planet_img)]
	local comment = comments['xxx01'][rnd:random(#comments['xxx01'])]..' is '..comments['xxx02'][rnd:random(#comments['xxx02'])]
	local r_name = ''
	repeat
		r_name = ''
		for _ = 1, rnd:random(2, 4) do
			r_letter = rnd:random(r_letters:len() /3)
			r_name = r_name..r_letters:sub(r_letter * 3 - 2, r_letter * 3 - 2)..r_letters:sub(r_letter * 3 - 1, r_letter * 3 - 1)..r_letters:sub(r_letter * 3, r_letter * 3)
		end
		r_name = r_name:gsub(' ', '')
		comment = comment:gsub('xxxrand', r_name, 1)
		for k1, k2 in pairs(comments) do
			comment = comment:gsub(k1, k2[rnd:random(#k2)], 1)
		end
	until comment:find('xxx') == nil
	comment = comment:gsub('pppnian', (name..'xxx'):gsub('ixxx', ''):gsub('exxx', ''):gsub('xxx', '')..'ian')
	comment = comment:gsub('pppn', name)
	comment = comment:gsub('  ', ' ')
	rnd:randomseed(rnd:random(1000000) * os.time())
	local p_angle = rnd:random() * math.pi * 2
	local pos, angle = vector(d * math.cos(p_angle), d * math.sin(p_angle)), rnd:random() * 2 * math.pi
	self.name = name
	self.pos = pos
	self.angle = angle
	self.img = img
	self.scale = scale
	self.color = {r, g, b, 255}
	self.off = self.img:getHeight() / 2
	self.comment = comment
	self.draw_dist = 100000^2
end

function planet:draw()
	if (self.pos - player.pos):len2() > self.draw_dist then return end
	love.graphics.setColor(self.color)
	love.graphics.draw(self.img, self.pos.x, self.pos.y, self.angle, self.scale, self.scale, self.off, self.off)
end

sun = class('sun')

function sun:initialize(ss)
	self.ss = ss
	self.name = u.ss_name
	rnd:randomseed(ss * 10)
	self.pos, self.scale, r, g, b = vector(0, 0), 1 + rnd:random() * 2, rnd:random(64, 256), rnd:random(64, 256), rnd:random(64, 256)
	self.color = {r, g, b, 255}
	self.draw_dist = 100000^2
	self.off = 4096
end

function sun:draw()
	if (self.pos - player.pos):len2() > self.draw_dist then return end
	love.graphics.setColor(self.color)
	love.graphics.circle("fill", self.pos.x, self.pos.y, self.scale * self.off, 90)
end
