-------------------------------------------------
-- Eternity alpha 0.5
-- Licence: GPL
-- Copyright (c) 2011 Caldas Lopes aka Linux-man
-------------------------------------------------

player_ship = class('player_ship', ship)

function player_ship:initialize(pos, v, model)
	ship.initialize(self, pos, v, model)
	self.type = 'player'
	self.cash = 100000
	self.down_dt = 1
	self.down_key = ''

end

function player_ship:keyreleased(key, unicode)
	if self.docked then
		if key == ' ' then self:launch() end
	else
		if u.time_delta == 1 and not u.jump then
			if key == 'a' then self.autopilot = not self.autopilot end
			if not self.autopilot then
				if key == 'j' then self:jump() end
				if key == 'n' then self:change_gun_active() end
				if key == 'm' then self:missile() end
				if key == 'down' and self.down_key == 'down' and self.down_dt < 0.3 then self.a = 0 end
				if key == 'up' and self.down_key == 'up' and self.down_dt < 0.3 and self.e_turbo then self.turbo = 1 end
				if key == 't' and self.e_targeting_system and self.e_scanner > 0 then
					if self.down_key == 't' and self.down_dt < 0.3 then self.target = nil
					else self:acquire_target(nil) end
				end
			end
		end
		self.down_dt = 0
		self.down_key = key
	end
end

function player_ship:update(dt)
	if self.destroyed and not self.engine:isActive() then u:initialize() end

	if u.time_delta == 1 and not self.autopilot and not self.docked and not u.jump then
		if love.keyboard.isDown('right') then
			self.angle = self.angle + (self.turn * dt)
		elseif love.keyboard.isDown('left') then
			self.angle = self.angle - (self.turn * dt)
		end
		if love.keyboard.isDown('down') then
			self.turbo = 0
			self.a = math.max(- self.a_max / 10, self.a - (self.aa * dt))
		elseif love.keyboard.isDown('up') then
			self.a = math.min(self.a_max, self.a + (self.aa * dt))
		end
		if love.keyboard.isDown(' ') then self:fire() end
	end
	ship.update(self, dt)
	self.down_dt = self.down_dt + dt
end

function player_ship:draw()
	if self.docked then return end
	ship.draw(self)
	if self.destroyed == false then
		if self.target then love.graphics.circle('line', self.target.pos.x, self.target.pos.y, 64) end
		if u.scale < 0.5 and u.scan_mode > 0 then
			love.graphics.setColor(255 ,255, 255, 128)
			love.graphics.line(self.pos.x - math.cos(self.angle - math.pi / 6) * 6 / u.scale, self.pos.y - math.sin(self.angle - math.pi / 6) * 6 / u.scale, self.pos.x, self.pos.y, self.pos.x - math.cos(self.angle + math.pi / 6) * 6 / u.scale, self.pos.y - math.sin(self.angle + math.pi / 6) * 6 / u.scale)
		end
	end
end

function player_ship:destroy()
	u.time_delta = 1
	ship.destroy(self)
end

function player_ship:autopilot_0(dt)
	ship.autopilot_0(self, dt)
	if self.autopilot then
		if u.time_delta > 10 then
			if (self.destiny[1] - self.pos):len2() < 100000^2 then u.time_delta = 10 end
		end
		if (self.destiny[1] - self.pos):len2() < 20000^2 then
			u.time_delta = 1
			table.remove(self.destiny, 1)
		end
	end
end

function player_ship:allow_jump()
	return self.e_hyperspace > 0 and u.ss_galaxy > 0 and u.ss ~= u.ss_galaxy and self.fuel > self.fuel_max * u.ss_distance / 20 * self.e_hyperspace and self.pos:len() > 1000000 * u.galaxy[u.ss].scale
end

function player_ship:jump()
	if self:allow_jump() then
		self.fuel = self.fuel - self.fuel_max * u.ss_distance / 20 * self.e_hyperspace
		u.ss = u.ss_galaxy
		u.jump = true
		u.jump_countdown = 1200 * u.ss_distance / 20 * self.e_hyperspace
	end
end

function player_ship:change_gun_active()
	if self.gun_active == '' then self.gun_active = 'front'	
	elseif self.gun_active == 'front' then self.gun_active = 'rear'
	elseif self.gun_active == 'rear' then self.gun_active = 'left'
	elseif self.gun_active == 'left' then self.gun_active = 'right' 
	elseif self.gun_active == 'right' then self.gun_active = 'auto'
	else self.gun_active = 'front' end
	self:gun_test()
end

function player_ship:gun_test()
	if self.gun_active == 'front' and self.gun_front == nil then self.gun_active = 'rear' end
	if self.gun_active == 'rear' and self.gun_rear == nil then self.gun_active = 'left' end
	if self.gun_active == 'left' and self.gun_left == nil then self.gun_active = 'right' end
	if self.gun_active == 'right' and self.gun_right == nil then self.gun_active = 'auto' end
	if self.gun_active == 'auto' and self.gun_auto == nil then self.gun_active = 'front' end
	if self.gun_front == nil and self.gun_rear == nil and self.gun_left == nil and self.gun_right == nil and self.gun_auto == nil then self.gun_active = '' end
end
function player_ship:launch()
	self:gun_test()
	self.docked = false
	u.view = cockpit.create()
	if self.dock.type == 'space_station' then
		self.angle = theta(self.dock.v:unpack())
		self.pos = self.dock.pos + self.dock.v
		self.v = self.dock.v
		self.a =self.a_max
	elseif self.dock.type == 'ground_station' then
		self.angle = self.dock.angle
		self.pos = self.dock.pos + (self.dock.beam + (self.off * self.scale)) * vector(math.cos(self.dock.angle), math.sin(self.dock.angle))
		self.v = self.v_max / 2 * vector(math.cos(self.dock.angle), math.sin(self.dock.angle))
		self.a =self.a_max
	elseif self.dock.type == 'ring_station' then
		self.angle = rnd:random() * math.pi * 2
		self.pos = self.dock.pos + ((self.dock.off * self.dock.scale) + (self.off * self.scale)) * vector(math.cos(self.angle), math.sin(self.angle)) * 1.01
		self.v = self.v_max / 2 * vector(math.cos(self.angle), math.sin(self.angle))
		self.a =self.a_max
	end
end
