-------------------------------------------------
-- Eternity alpha 0.5
-- Licence: GPL
-- Copyright (c) 2011 Caldas Lopes aka Linux-man
-------------------------------------------------

ship = class('ship')

function ship:initialize(pos, v, model)
	local particlelife, r, g, b
	self.aa, self.a_max, self.v_max, self.turn, self.img, self.scale, self.fuel_delta, self.fuel_max,
	self.hull_delta, self.hull_max, self.cargo_max, self.missiles_max, self.mines_max,
	self.gun_front_max, self.gun_rear_max, self.gun_left_max, self.gun_right_max, self.gun_auto_max,
	particlelife, r, g, b = model()
	self.destroyed = false
	self.type = 'ship'
	self.status = 'legal'
	self.dock = nil
	self.docked = false
	self.pos = pos
	self.name = string.char(rnd:random(65, 90), rnd:random(97, 122), rnd:random(97, 122), 45, rnd:random(48, 57), rnd:random(48, 57))
	self.color = {rnd:random(128, 255), rnd:random(128, 255), rnd:random(128, 255), 255}
	self.turn = 2 * math.pi * self.turn --Turn rate
	self.off = self.img:getHeight()/2 -- shipoffset: think square ships, so we dont need to have 2 offsets
	self.angle = 0
	self.temp = 0
	self.destiny = {}
	self.autopilot = false
	self.a = 0 --accel
	self.accel = vector(0, 0)
	self.gun_charge = 1
	self.gun_delta = 0
	self.v = v
	self.dir = vector(1, 0) --angle in cartesian coord
	self.fuel = self.fuel_max
	self.turbo = 0
	self.hull = self.hull_max
	self.draw_dist =  10000^2
	self.update_dist = 1200000^2
	self.destroy_dist = 1100000^2
	self.engine = love.graphics.newParticleSystem(eng_img, 1000)
	self.engine:setLifetime(-1)
	self.engine:setSpread(0.5)
	self.engine:setParticleLife(particlelife * 0.75, particlelife * 1.25)
	self.engine:setSize(1, 1.5, 0)
	self.engine:setColor( 255, 255, 255, 255, r, g, b, 128 )
	self.target = nil
	self.cargo = {}
	for _,o in pairs (commodities) do table.insert(self.cargo, {n = o.n, c = o.c, q = 0, u = o.u}) end
	self.cargo_t = 0
	self.e_hyperspace = 1
	self.e_autopilot = 1
	self.e_scanner = 1
	self.e_passenger_berth = 0
	self.e_energy_bomb = 0
	self.e_cargo_bay_extension = false
	self.e_comm_system = false
	self.e_hull_booster = false
	self.e_ecm_system = false
	self.e_turbo = false
	self.e_scoops = false
	self.e_targeting_system = false
	self.e_fuel_collector = false
	self.e_auto_aim = false
	
	self.missiles_mk1 = self.missiles_max
	self.missiles_mk2 = 0
	self.missiles_mk3 = 0
	self.ecm_missiles = 0
	self.missiles = self.missiles_max
	self.mines_mk1 = self.mines_max
	self.mines_mk2 = 0
	self.mines_mk3 = 0
	self.mines = self.mines_max
	self.gun_active = 'front'
	self.gun_front = laser(self, 'Laser Mk I', 'front', 1)
	self.gun_rear = nil
	self.gun_left = nil
	self.gun_right = nil
	self.gun_auto = nil
	self.firing = 0
end

function ship:acceleration()
	if self.fuel == 0 then return 0
	else return self.a * (1 - self.turbo) + self.a_max * self.turbo * 10 end
end

function ship:update(dt)
	if self.docked then		
		self.pos = self.dock.pos
		self.v = self.dock.v
		self.accel = vector(0, 0)
		return
	end
	if (self.pos - player.pos):len2() > self.destroy_dist then self:destroy() end
	if (self.pos - player.pos):len2() > self.update_dist then return end
	if self.destroyed then
		self.v = vector(0, 0)
		self.engine:update(dt)
		return
	end
	if self.autopilot then self:autopilot_0(dt)	end
	if self.target then
		if self.target.destroyed or (self.pos - self.target.pos):len2() > 100000^2 then self.target = nil end
	end
	
	self.dir = vector(math.cos(self.angle), math.sin(self.angle))

	self.accel = self:acceleration() * self.dir

	self.engine:setEmissionRate(math.abs(math.min(400, self.accel:len() /self.a_max * 200))) --variable rate
	self.engine:setSpeed(math.abs(math.min(800, self.accel:len() / self.a_max * 400))) --variable speed
	self.engine:setDirection(theta((- self.accel):unpack())) ---self.accel
	self.engine:setPosition((- self.off * self.scale * self.dir):unpack())
	self.engine:update(dt)
	
	for _,o in pairs(u.planets) do
		if (o.pos-self.pos):len2() < 100000^2 then
			self.accel = self.accel + (o.pos-self.pos):normalized() * u.G * o.off * o.scale /  (o.pos-self.pos): len2() --gravity
			if (o.pos-self.pos):len2() < ((o.scale * o.off) + (self.scale * self.off))^2 then self:destroy() end
		end
	end
	if (u.planets[0].pos-self.pos):len2() < 100000^2 then
		self.temp = self.temp + (100000 - (u.planets[0].pos-self.pos):len()) * dt / 1000000 --heat
	end
	self.dock = nil
	self.docked = false
	for _,o in pairs(u.obj) do
		if (o.pos-self.pos):len2() < 10000^2 and o.type:find('station') then
			self.dock = o
			if ((self.dock.type == 'space_station' or self.dock.type == 'ring_station')
			and (self.dock.pos-self.pos):len2() < ((self.dock.scale * self.dock.off) + (self.scale * self.off))^2
			and (self.v - self.dock.v):len2() < self.dock.dock_v^2)
			or
			(self.dock.type == 'ground_station'
			and (self.dock.pos-self.pos):len2() < self.dock.beam^2
			and (self.v - self.dock.v):len2() < self.dock.dock_v^2
			and theta((self.pos - self.dock.pos):unpack()) < self.dock.angle + math.pi / 8
			and theta((self.pos - self.dock.pos):unpack()) > self.dock.angle - math.pi / 8)
			then self.docked = true end
		end
		if not self.docked and (o.pos-self.pos):len2() < 2000^2 and o.pos ~= self.pos and o.destroyed == false then
			if (o.pos-self.pos):len2() < ((o.scale * o.off) + (self.scale * self.off))^2 then self:hit(o.hull) end
		end
	end
	self.temp = math.max(0, self.temp - dt / 100) --cooling
	if self.temp > 100 then self:destroy() end --destroyed by heat
	self.v = self.v + self.accel * dt
	self.pos = self.pos + self.v * dt
	self.gun_charge = math.min(1, self.gun_charge + self.gun_delta * dt) --charging guns
	if self.v:len2() > self.v_max^2 then self:hit((self.v:len() - self.v_max) * 0.0001 * dt) end --excessive velocity
	if self.e_hull_booster and self.hull < self.hull_max then self.hull = self.hull + self.hull_delta * dt * 9 self.fuel = self.fuel - self.fuel_delta * 4 end
	self.hull = math.min (self.hull_max, self.hull + self.hull_delta * dt) --hull recovery
	self.fuel = math.max(0, self.fuel - self.fuel_delta * self.accel:len() * (1 + self.turbo * 2) * dt) --fuel consumption
end

function ship:draw()
	if self.docked then return end
	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.push()
	love.graphics.translate((self.pos):unpack())
	love.graphics.draw(self.engine, 0, 0)
--[[	if self.destroyed == false and player.scan_mode > 3 then --hull bar
		love.graphics.rectangle('fill', 0, 0 - 2 * self.off, 20, 6)
		love.graphics.setColor(255 ,0, 0, 255)
		love.graphics.rectangle('fill', 0, 0 - 2 * self.off, self.hull / self.hull_max * 20, 6)
		love.graphics.setColor(255 ,255, 255, 255)
	end ]]--
	love.graphics.pop()
	if self.destroyed == false then
		if self.firing > 0 then
			if self.gun_active == 'front' then self.gun_front:draw()
			elseif self.gun_active == 'rear' then self.gun_rear:draw()
			elseif self.gun_active == 'left' then self.gun_left:draw()
			elseif self.gun_active == 'right' then self.gun_right:draw()
			elseif self.gun_active == 'auto' then self.gun_auto:draw() end
		end
		love.graphics.setColor(self.color)
		--love.graphics.setColor(255 ,255, 255, 255)
		love.graphics.draw(self.img, self.pos.x, self.pos.y, self.angle, self.scale, self.scale, self.off, self.off)
	end
end

function ship:hit(hit)
	self.hull = math.max(0, self.hull - hit)
	if self.hull == 0 then self:destroy() end
end

function ship:destroy()
	if self.destroyed == false then
		self.destroyed = true
		self.engine:reset()
		self.engine:setLifetime(1)
		self.engine:setDirection(0)
		self.engine:setEmissionRate(1000)
		self.engine:setSpeed(300 , 400)
		self.engine:setSize(1, 5)
		self.engine:setColor(255, 255, 255, 255, 255, 0, 0, 0)
		self.engine:setParticleLife(1)
		self.engine:setSpread(math.pi * 2)
		self.engine:setPosition(0, 0)
	end
end

function ship:fire()
	if self.gun_active == 'front' then self.gun_front:fire()
	elseif self.gun_active == 'rear' then self.gun_rear:fire()
	elseif self.gun_active == 'left' then self.gun_left:fire()
	elseif self.gun_active == 'right' then self.gun_right:fire()
	elseif self.gun_active == 'auto' then self.gun_auto:fire() end
end

function ship:autopilot_0(dt)
	if #self.destiny == 0 then
		self.autopilot = false
		return
	end
	self.angle = theta(self.dir:unpack())
	local destiny = self.destiny[1]
	local angle = theta((destiny - self.pos):unpack())
	if math.abs(angle-self.angle) < self.turn * dt then self.angle = angle
	elseif ((math.abs(angle-self.angle)<math.pi and angle-self.angle>0)
	or (math.abs(angle-self.angle)>math.pi and angle-self.angle<0))
	then self.angle = self.angle + (self.turn * dt)
	else self.angle = self.angle - (self.turn * dt) end

	if self.v * self.dir > self.v_max or (destiny - self.pos):len2() < (1 / 2 * self.v * self.dir * self.v * self.dir / (self.a_max / 10))^2
	then self.a = math.max(- self.a_max / 10, self.a - (self.aa * dt))
	elseif self.v * self.dir > 0.9 * self.v_max then self.a = 0
	else self.a = math.min(self.a_max, self.a + (self.aa * dt)) end
	--x = x0 + v0 * t - 1/2 * a t^2   v  = v0 - at t =v0 / a x = v0*v0/a+1/2*a*(v0/a)^2 = v*v/a - 1/2 * v*v/a = 3/2 * v*v/a
end

function ship:acquire_target(target)
	if target then
		self.target = target
	else
		if self.target == nil then
			for _,o in pairs(u.obj) do
				if (o.pos-self.pos):len2() < 100000^2 and o.pos ~= self.pos and o.destroyed == false then
					if self.target == nil then self.target = o
					else
						if (o.pos-self.pos):len2() < (self.target.pos-self.pos):len2() then self.target = o end
					end
				end
			end
		else
			local t = nil
			for _,o in pairs(u.obj) do
				if (o.pos-self.pos):len2() < 100000^2 and o.pos ~= self.pos and o.destroyed == false then
					if t == nil and (o.pos-self.pos):len2() > (self.target.pos-self.pos):len2() then t = o end
					if t and (o.pos-self.pos):len2() > (self.target.pos-self.pos):len2() and (o.pos-self.pos):len2() < (t.pos-self.pos):len2() then t = o end
				end
			end
			if t == nil then 
				self.target = nil
				self:acquire_target(nil)
			else
				self.target = t
			end
		end
	end
end

function ship:missile()
		local lpos, lvel = self.pos + self.dir * self.off * 2, self.v + self.dir * 100
		table.insert(u.shots, missile(lpos, lvel, self.dir, self.target, 10, {255, 255, 255, 255}))
end

space_station = class('space_station', object)

function space_station:initialize(planet_n)
	rnd:randomseed(u.ss * 10 + planet_n)
	local name = 'Space Station'
	local angle =  rnd:random () * 2 * math.pi - math.pi
	local scale = 1
	local hull = 1000
	local r, g, b = rnd:random(128, 255), rnd:random(128, 255), rnd:random(128, 255)
	self.img = img
	self.planet_n = planet_n
	local o = u.planets[planet_n]
	local d = o.scale * o.off * 2 + rnd:random() * o.scale * o.off * 8
	local pos = o.pos + vector(d, 0)
	local v = vector(0, math.sqrt(u.G * o.off * o.scale / d))
	local img = station_img[rnd:random(#station_img)]
	object.initialize(self, name, pos, v, hull, angle, scale, r, g, b, img)
	--self.draw_dist = 10000^2
	--self.update_dist = 1000000^2
	self.destroy_dist = 100000000^2
	self.hull_max = hull
	self.type = 'space_station'
	self.status = 'legal'
	self.dock_v = 50
end

function space_station:update(dt)
	if u.time_delta > 10 then return end
	if (self.pos - player.pos):len2() > self.update_dist then return end
	local o = u.planets[self.planet_n]
	self.accel = (o.pos-self.pos):normalized() * u.G * o.off * o.scale /  (o.pos-self.pos): len2() --gravity
	self.v = self.v + self.accel * dt
	object.update(self, dt)
	self.angle = self.angle + math.abs(self.angle) / self.angle * rnd:random() / 2 * dt
end

ground_station = class('ground_station', object)

function ground_station:initialize(planet_n)
	rnd:randomseed(u.ss * 10 + planet_n)
	local name = 'Ground Station'
	local o = u.planets[planet_n]
	local d, angle = o.scale * o.off, rnd:random() * math.pi * 2
	local pos = o.pos + vector(d * math.cos(angle), d * math.sin(angle))
	local v = vector(0, 0)
	local hull = 1000
	local scale = 1
	local r, g, b = rnd:random(128, 255), rnd:random(128, 255), rnd:random(128, 255)
	object.initialize(self, name, pos, v, hull, angle, scale, r, g, b, nil)
	--self.draw_dist = 10000^2
	--self.update_dist = 1000000^2
	self.destroy_dist = 100000000^2
	self.hull_max = hull
	self.type = 'ground_station'
	self.status = 'legal'
	self.dock_v = 200
	self.off = 16
	self.beam = 256 + rnd:random() * 512
end

function ground_station:draw()
	love.graphics.setColor(self.color)
	love.graphics.circle('fill', self.pos.x, self.pos.y, self.off)
	self.engine:setColor(255, 255, 255, 255)
	if self.destroyed == false then
		for n = self.off, self.beam, 4 / u.scale do
			love.graphics.setColor(self.color[1] ,self.color[2], self.color[3], 255 - n * 255 / self.beam )
			love.graphics.point(self.pos.x + math.cos(self.angle) * n + (rnd:random(8) - 4), self.pos.y + math.sin(self.angle) * n + (rnd:random(8) - 4))
		end
	end
	love.graphics.draw(self.engine, self.pos.x, self.pos.y)
end

ring_station = class('ring_station', object)

function ring_station:initialize(planet_n)
	rnd:randomseed(u.ss * 10 + planet_n)
	local name = 'Ring Station'
	local o = u.planets[planet_n]
	local angle = rnd:random() * math.pi * 2
	local pos = o.pos
	local v = vector(0, 0)
	local hull = 10000
	local scale = o.scale * (1 + rnd:random())
	local r, g, b = rnd:random(64, 128), rnd:random(64, 128), rnd:random(64, 128)
	local img = ring_img
	object.initialize(self, name, pos, v, hull, angle, scale, r, g, b, img)
	--self.draw_dist = 10000^2
	--self.update_dist = 1000000^2
	self.destroy_dist = 100000000^2
	self.hull_max = hull
	self.type = 'ring_station'
	self.status = 'legal'
	self.dock_v = 100
	self.lights = {}
	for n = 1, 30 do
		local angle = rnd:random() * math.pi * 2
		table.insert(self.lights, {pos = self.pos + vector((self.scale * self.off - rnd:random(32))* math.cos(angle), (self.scale * self.off - rnd:random(32)) * math.sin(angle)),
			color = {128 + rnd:random(127), 128 + rnd:random(127), 128 + rnd:random(127), 255}, on = true})
	end
end

function ring_station:update(dt)
	object.update(self, dt)
	for _,o in pairs(self.lights) do
		if rnd:random(5000) < 10 then o.on = not o.on end
	end
	
end

function ring_station:draw()
	object.draw(self)
	for _,o in pairs(self.lights) do
		if o.on then
			love.graphics.setColor(o.color)
			love.graphics.draw(phaser_img, o.pos.x, o.pos.y, 0, 5, 5, phaser_img:getHeight()/2, phaser_img:getHeight()/2)
			--love.graphics.circle('fill', o.pos.x, o.pos.y, 8)
		end
	end
	love.graphics.setColor(255, 255, 255, 255)
end
