-------------------------------------------------
-- Eternity alpha 0.5
-- Licence: GPL
-- Copyright (c) 2011 Caldas Lopes aka Linux-man
-------------------------------------------------

require('data.lua')
require('objects.lua')
require('ships.lua')
require('ship.lua')
require('player.lua')
require('background.lua')
require('guns.lua')
require('views.lua')

function Serialize(t)
	if not t then return nil end
	local ko, vo
	local s = "{"
	for k, v in pairs(t) do
		ko, vo = nil, nil
		if type(k) == "string" then ko = k
		elseif type(k) == "number" then ko = "["..k.."]"
		end
		if type(v) == "number" then vo = v
		elseif type(v) == "string" then vo = ("%q"):format(v)
		elseif type(v) == "boolean" then vo = v and "true" or "false"
		elseif type(v) == "userdata" then vo = ("%q"):format(tostring(v))
		elseif type(v) == "table" then vo = Serialize(v)
		end
		if ko and vo then s = s..ko.."="..vo.."," end
	end
	return s.."}"
end

function theta(x, y) --cartesian-polar conversion for theta
	if x >= 0 then return math.atan(y/x)
	elseif y < 0 then return math.atan(y/x)-math.pi
	else return math.atan(y/x)+math.pi end
end

function create_name()
	local name = ''
	for _ = 1, rnd:random(2, 4) do name = name .. s[rnd:random(32)] end
	name = name:gsub("^%l", string.upper)
	return name
end

space = class('space')

function space:initialize()
	self.commander = 'Jameson'
	self.scale = 1
   	self.scanner_scale = 0.01
	self.scan_mode = 0
	self.hud_scale = 100000
	self.ss_galaxy = 0
	self.show_planet = -1	
	self.center = vector(w / 2, h / 2)
	self.time_delta = 1
	self.time = 0
	self.paused = false
	self.G = 100000
	self.obj = {}
	self.rays = {}
	self.nebulas = {}
	self.stars = {}
	self.planets = {}
	self.ast_fields = {}
	self.shots = {}
	self.galaxy = {}
	self.market = {}
	self:create_galaxy()
	self.jump = false
	self.ss_distance = 0
	self.jump_countdown = 0
	self.menu = 'cockpit'
	self.view = cockpit.create()
	self.ss = 512
	self.scanner_range = 1
end

function space:keyreleased(key, unicode)
	if key == 'escape' then love.event.push('q') end
	if key == 'p' then self.paused = not self.paused end
	if self.paused then return end
	if key == 'q' then self.obj[0] = nil end
	if     key == '1' then self.view = cockpit.create()
	elseif key == '2' then self.view = scanner.create()
	elseif key == '3' then self.view = galaxy.create()
	elseif key == '4' then self.view = market.create()
	elseif key == '5' then self.view = equipment.create()
	elseif key == '6' and player.docked then self.view = ships.create()
	elseif key == '7' and player.docked then self.view = billboard.create()
	elseif key == '8' then self.view = options.create()
	end
	if key == '9' then self.time_delta = math.max(1, self.time_delta / 10)
	elseif key == '0' and (player.a == 0 or player.autopilot) then self.time_delta = math.min(100, self.time_delta * 10) end
	if key == 's' then
		self.scan_mode = self.scan_mode + 1
		if self.scan_mode > 4 then self.scan_mode = 0 end
	end
	if key == 'c' then
		self.hud_scale = self.hud_scale / 10
		if self.hud_scale < 10000 then self.hud_scale = 100000 end
	end
	self.view:keyreleased(key, unicode)
	player:keyreleased(key, unicode)
end

function space:mousepressed(x, y, button)
	if self.paused then return end
	self.view:mousepressed(x, y, button)
end

function space:update(dt)
	local ship_nr, asteroid_nr = 0, 0
	if self.paused then dt = 0 return end
	if self.obj[0] == nil then
		self:create_ss(self.ss, 0, 0)
	end
	if self.time_delta > 1 then
		for _,o in pairs(self.planets) do
			if (o.pos-player.pos):len2() < 100000^2 then
				self.time_delta = 10
				if (o.pos-player.pos):len2() < 20000^2 then self.time_delta = 1 end
			end
		end
		if #self.shots > 0 then self.time_delta = 1 end
	end
	dt = dt * self.time_delta
	self.time = self.time + dt
	if self.jump then
		if self.jump_countdown > 0 then self.jump_countdown = self.jump_countdown - dt
		else
			rnd:randomseed()
			self.time_delta = 1
			local d, angle = 1000000 * self.galaxy[self.ss].scale + rnd:random(1000000), rnd:random() * math.pi * 2
			self:create_ss(self.ss, d * math.cos(angle), d * math.sin(angle))
		end
	end
   	for _,o in pairs(self.shots) do
		if o.destroyed and not o.engine:isActive() then table.remove(self.shots, _) end
	end
	for _,o in pairs(self.obj) do
		if o.destroyed and not o.engine:isActive() then table.remove(self.obj,_) end
		if o.type == 'ship' then ship_nr = ship_nr + 1 end
		if o.type == 'asteroid' then asteroid_nr = asteroid_nr + 1 end
	end
	local bg = 46 - math.min(1, player.pos:len() / 1000000) * 46
	love.graphics.setBackgroundColor(bg, bg, bg)
	if asteroid_nr < self.debris then
		for n = 1, self.debris - asteroid_nr do
			local d, angle = 1000000 + rnd:random(100000), rnd:random() * math.pi * 2
			table.insert(self.obj, asteroid(player.pos + vector(d * math.cos(angle), d * math.sin(angle))))
		end
	end
	for _,o in pairs(self.ast_fields) do --asteroid fields
		if (o.pos - player.pos):len2() > 1600000^2 then o.render = false end
		if (o.pos - player.pos):len2() < 1200000^2 and o.render == false then
			o.render = true
			for n = 1, rnd:random(50, 100) do
				local d, angle = rnd:random(o.r), rnd:random() * math.pi * 2
				table.insert(self.obj, field(o.pos + vector(d * math.cos(angle), d * math.sin(angle))))
			end
		end
	end
	for _,o in pairs(self.obj) do o:update(dt) end
   	for _,o in pairs(self.shots) do o:update(dt) end
	for _,o in pairs(self.nebulas) do o:update(dt) end
	for _,o in pairs(self.rays) do o:update(dt) end
	for _,o in pairs(self.stars) do o:update(dt) end

	self.view:update(dt)

	--if #self.obj < 40 then
	--	for n = 1, 40 - #self.obj do
	--		local d, angle = 100000 + rnd:random(900000), rnd:random() * math.pi * 2
	--		table.insert(self.obj, ship(player.pos + vector(d* math.cos(angle), d * math.sin(angle)), player.v, 2, 20, 1000, 0.25, adder))
	--	end
	--end

	--for _,o in pairs(self.obj) do
		--if o.type == 'ship' then
		--	o.destiny[1] = player.pos
		--	o.autopilot = true
		--end
		--o:update(dt)
	--end
end

function space:draw()
	self:draw_background()
   	love.graphics.push()
	love.graphics.translate((self.center - player.pos * self.scale):unpack())
	love.graphics.scale(self.scale)
	for _,o in pairs(self.nebulas) do o:draw() end
	for _,o in pairs(self.obj) do o:draw() end
	for _,o in pairs(self.shots) do o:draw() end
	for _,o in pairs(self.planets) do o:draw() end
 	love.graphics.pop()
	love.graphics.setColor(255, 255, 255, 255)
	self.view:draw()
	self:draw_instruments()
	if self.jump then
		love.graphics.setColor(255, 255, 255, ((1200 * self.ss_distance / 20 * player.e_hyperspace) - self.jump_countdown) / (1200 * self.ss_distance / 20 * player.e_hyperspace) * 255)
		love.graphics.rectangle('fill', 0, 0, w, h)
	end
	if self.paused then
		love.graphics.setFont(frontier2)
		love.graphics.printf('PAUSED', 0, h / 2, w, "center")
		love.graphics.setFont(frontier)
	end

	love.graphics.print('fps: '..love.timer.getFPS(), 0, 0)
end

function space:create_ss(ss, x, y)
	self.jump = false
	self.ss_galaxy = 0
	self.ss_name, self.ss_pos, self.planets_n, self.specie, self.gov, self.econ, self.tech, self.pop, self.prod, self.corr = self:calc_ss(ss)
	self.planets = {}
	self.planets[0] = sun(ss)
	for n = 1, self.planets_n do table.insert(self.planets, planet(self.ss, self.planets_n, n)) end
	self:calc_market()
	self.debris = rnd:random(20)
	if x == 0 then
		self.obj = {}
		self.obj[0] = player_ship(vector(x, y), vector(0, 0), cobra)
		if self.planets[1] then table.insert(self.obj, ring_station(1)) end
		if self.planets[2] then table.insert(self.obj, space_station(2)) end
		if self.planets[3] then table.insert(self.obj, ground_station(3)) end
		self.obj[0].docked = true
		self.obj[0].dock = self.obj[1]
	else
		while #self.obj > 1 do table.remove(self.obj) end
		if self.planets[1] then self.obj[1] = space_station(1, station_img) end
		if self.planets[2] then self.obj[2] = ground_station(2, station_img) end
		if self.planets[3] then self.obj[3] = ring_station(3, station_img) end
		self.obj[0].pos = vector(x, y)
	end
	player = self.obj[0]
	rnd:randomseed(rnd:random(1000000) * os.time())
	local ast_field_n = rnd:random(4)
	self.ast_fields = {}
	for n = 1, ast_field_n do
		local d, angle = 2000000 + rnd:random(2000000), rnd:random() * math.pi *2
		table.insert(self.ast_fields, {pos = vector(d * math.cos(angle), d * math.sin(angle)),
			r = 100000 + rnd:random(100000), render = false})
	end
	for n = 1, rnd:random(self.debris) do
		local d, angle = 100000 + rnd:random(900000), rnd:random() * math.pi * 2
		table.insert(self.obj, asteroid(player.pos + vector(d * math.cos(angle), d * math.sin(angle))))
	end
	self.rays = {} self.nebulas = {} self.stars = {}
	for n = 1, 60 do table.insert(self.stars, star.create(player.pos.x, player.pos.y)) end
	for n = 1, 30 do table.insert(self.rays, ray.create(player.pos.x, player.pos.y)) end
	for n = 1, 10 do table.insert(self.nebulas, nebula.create(player.pos.x, player.pos.y)) end
end

function space:calc_ss(ss)
	rnd:randomseed(ss * 10)
	local ss_name = create_name()
	local ss_pos = vector(rnd:random(-200, 200), rnd:random(-150, 150))
	local planets_n = 1 + rnd:random(4)
	local specie
	if rnd:random(2) == 1 then specie = 'Human Colonists'
	else specie = inhabit1[rnd:random(4)]..inhabit2[rnd:random(7)]..inhabit3[rnd:random(7)]..species[rnd:random(8)]	end
	local gov = rnd:random(8)
	local econ
	if gov < 3 then econ = rnd:random(2) * 4 - rnd:random(0,1)
	else econ = rnd:random(8) end
	local tech = 8 - econ + math.floor(gov / 2) + rnd:random(3)
	local pop = (tech + (gov - 1) + (econ - 1)) / 10
	local prod = (pop * 10 + (tech - 1) * 10 + (gov - 1) * 5 + rnd:random(10))^2
	local corr = 1
	if rnd:random(gov * 2) == 1 then corr = -1 end
	return ss_name, ss_pos, planets_n, specie, gov, econ, tech, pop, prod, corr
end

function space:calc_market()
	rnd:randomseed()
	self.market = {}
	for commodity, values in pairs(commodities) do
		if values.c * self.corr >= 0 then
			local changing = rnd:random(0,values.m)
			local product = (self.econ - 1) * values.g
			local q = math.max(0, values.q + changing - product)
			if q > 255 then q = q - 255 end
			local p = (values.p + changing + product) / 4
			if values.n == 'Alien Items' then q = 0 end
			self.market[commodity] = {n = values.n, q = q, p = p, u = values.u, c = values.c}
		end
	end
end

function space:create_galaxy()
	local x, y
	for n = 1, 1024 do
		local ss_name, ss_pos, planets_n, specie, gov, econ, tech, pop, prod, corr = self:calc_ss(n)
		rnd:randomseed(n * 10)
		local scale, r, g, b = 1 + rnd:random() * 2, rnd:random(64, 256), rnd:random(64, 256), rnd:random(64, 256)
		local color = {r, g, b, 255}
		table.insert(self.galaxy, {ss_name=ss_name, ss_pos=ss_pos, planets_n=planets_n, specie=specie, gov=gov, econ=econ, tech=tech, pop=pop, prod=prod, corr=corr, scale=scale, color=color, x = 0, y = 0})
	end
end

function space:draw_background()
   	love.graphics.push()
	love.graphics.translate(self.center:unpack())
	for _,o in pairs(self.stars) do o:draw() end
	love.graphics.setPoint(1, 'smooth')
	love.graphics.setColor(255, 255, 255, math.min(128, player.v:len() / player.v_max * 128))
	for _,o in pairs(self.rays) do o:draw() end
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.pop()
end

function space:draw_instruments()
	local x0 = (w - 800) / 2
	love.graphics.setFont(frontier)
	love.graphics.setLine(1, 'rough')
	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.setPoint(1, 'smooth')	
	love.graphics.print('V', x0+10, h - 100 + 10)
	love.graphics.print(math.ceil(player.v:len()), x0+130, h - 100 + 10)
	if player.v:len2() > player.v_max^2 then love.graphics.setColor(255 ,0, 0, 255)
	else love.graphics.setColor(255 ,255, 255, 255) end
	love.graphics.rectangle('line', x0+20, h - 100 + 10, 100, 10)
	love.graphics.rectangle('fill', x0+20, h - 100 + 10, math.min(100, player.v:len() / player.v_max * 100), 9)

	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.print('A', x0+10, h - 100 + 25)
	love.graphics.print(math.ceil(player:acceleration()), x0+130, h - 100 + 25)
	if player.turbo == 1 then love.graphics.setColor(255 ,0, 0, 255)
	else love.graphics.setColor(255 ,255, 255, 255) end
	love.graphics.rectangle('line', x0+20, h - 100 + 25, 100, 10)
	if player.turbo == 0 and player.a < 0 then love.graphics.setColor(255 ,255, 0, 255) end
	love.graphics.rectangle('fill', x0+20, h - 100 + 25, math.min(1, math.abs(player:acceleration())/player.a_max) * 100, 9)

	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.print('F', x0+10, h - 100 + 40)
	love.graphics.print(math.ceil(player.fuel), x0+130, h - 100 + 40)
	if player.fuel<player.fuel_max / 4 then love.graphics.setColor(255 ,0, 0, 255)
	else love.graphics.setColor(255 ,255, 255, 255) end
	love.graphics.rectangle('line', x0+20, h - 100 + 40, 100, 10)
	love.graphics.rectangle('fill', x0+20, h - 100 + 40, player.fuel/player.fuel_max * 100, 9)

	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.print('H', x0+10, h - 100 + 55)
	love.graphics.print(math.ceil(player.hull), x0+130, h - 100 + 55)
	if player.hull < player.hull_max / 4 then love.graphics.setColor(255 ,0, 0, 255)
	else love.graphics.setColor(255 ,255, 255, 255) end
	love.graphics.rectangle('line', x0+20, h - 100 + 55, 100, 10)
	love.graphics.rectangle('fill', x0+20, h - 100 + 55, math.max(0, player.hull/player.hull_max) * 100, 9)

	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.print('L', x0+10, h - 100 + 70)
	love.graphics.print(player.gun_active, x0+130, h - 100 + 70)
	love.graphics.rectangle('line', x0+20, h - 100 + 70, 100, 10)
	love.graphics.rectangle('fill', x0+20, h - 100 + 70, player.gun_charge * 100, 9)

	love.graphics.print('T', x0+10, h - 100 + 85)
	if player.temp > 75 then love.graphics.setColor(255 ,0, 0, 255)
	else love.graphics.setColor(255 ,255, 255, 255) end
	love.graphics.rectangle('line', x0+20, h - 100 + 85, 100, 10)
	love.graphics.rectangle('fill', x0+20, h - 100 + 85, math.min(100, player.temp), 9)

	love.graphics.setColor(255 ,255, 255, 255)
	love.graphics.print('Zoom', x0+200, h - 100 + 10)
	love.graphics.printf(self.scale, x0+130, h - 100 + 10, 60, 'right')
	love.graphics.print('Time', x0+200, h - 100 + 40)
	love.graphics.printf(self.time_delta..'x', x0+130, h - 100 + 40, 60, 'right')
	love.graphics.print('X', x0+200, h - 100 + 55)
	love.graphics.printf(math.ceil(player.pos.x), x0+130, h - 100 + 55, 60, 'right')
	love.graphics.print('Y', x0+200, h - 100 + 70)
	love.graphics.printf(math.ceil(player.pos.y), x0+130, h - 100 + 70, 60, 'right')
	love.graphics.print('StarDate', x0+200, h - 100 + 85)
	love.graphics.printf(string.format("%.5f", self.time / 100000), x0+130, h - 100 + 85, 60, 'right')

	love.graphics.print('A', x0+250, h - 100 + 10)
	love.graphics.print('D', x0+250, h - 100 + 25)
	love.graphics.print('J', x0+250, h - 100 + 40)
	if not player.autopilot then love.graphics.setColor(0, 32, 0, 255)
	else love.graphics.setColor(0, 255, 0, 255) end
	love.graphics.circle('fill', x0+270, h - 100 + 15, 5)
	love.graphics.setColor(0, 32, 0, 255)
	if player.dock then 
		if (player.dock.v - player.v):len2() < 0.75 * player.dock.dock_v^2 then love.graphics.setColor(0, 255, 0, 255)
			elseif (player.dock.v - player.v):len2() < player.dock.dock_v^2 then love.graphics.setColor(255, 255, 0, 255)
			else love.graphics.setColor(255, 0, 0, 255)
		end
	end
	love.graphics.circle('fill', x0+270, h - 100 + 30, 5)
	love.graphics.setColor(0, 32, 0, 255)
	if self.jump then
		love.graphics.setColor(255, 0, 0, 255)
		love.graphics.print(math.ceil(self.jump_countdown), x0+280, h - 100 + 40)
	elseif player:allow_jump() then love.graphics.setColor(0, 255, 0, 255) end
	love.graphics.circle('fill', x0+270, h - 100 + 45, 5)
	self:draw_target()
	love.graphics.setColor(255 ,255, 255, 255)
end

function space:draw_target()
	local x0 = (w - 800) / 2
	if player.target then
		love.graphics.setColor(player.target.color)
		if player.target.img then love.graphics.draw(player.target.img, w - x0 - 5 - 90 + 44, h - 100 + 5 + 44, player.target.angle, 90 / (player.target.off * 2),  90 / (player.target.off * 2), player.target.off, player.target.off) end
		love.graphics.setColor(255 ,255, 255, 255)
		love.graphics.rectangle("line",  w - x0 - 5 - 90, h - 100 + 5, 90, 90)
		love.graphics.print('H  '..math.floor(player.target.hull), w - x0 - 5 - 90 - 50, h - 100 + 10)
		love.graphics.print('dV '..math.floor((player.target.v - player.v):len()), w - x0 - 5 - 90 - 50, h - 100 + 25)
		love.graphics.print('V  '..math.floor(player.target.v:len()), w - x0 - 5 - 90 - 50, h - 100 + 40)
		love.graphics.print('d  '..math.floor((player.target.pos - player.pos):len()), w - x0 - 5 - 90 - 50, h - 100 + 55)
		love.graphics.print('S '..player.target.status, w - x0 - 5 - 90 - 50, h - 100 + 70)
		love.graphics.setColor(255 ,255, 255, 128)
		love.graphics.line(w - x0 - 5 - 90 + 44, h - 100 + 5 + 44, w - x0 - 5 - 90 + 44 + math.min(40,player.target.v:len() / 4) * math.cos(theta(player.target.v:unpack())), h - 100 + 5 + 44 + math.min(40,player.target.v:len() / 4) * math.sin(theta(player.target.v:unpack())))
		love.graphics.setColor(255 ,255, 255, 255)
		love.graphics.line(w - x0 - 5 - 90 + 44, h - 100 + 5 + 44, w - x0 - 5 - 90 + 44 + math.min(40,(player.target.v - player.v):len() / 4) * math.cos(theta((player.target.v - player.v):unpack())), h - 100 + 5 + 44 + math.min(40,(player.target.v - player.v):len() / 4) * math.sin(theta((player.target.v - player.v):unpack())))
	end
end
